"""Timelink Python package.

Timelink, formerly known as MHK (Micro History with Kleio) is an information system
designed for processing person related information collected from historical sources.

.. moduleauthor:: Joaquim Ramos de Carvalho



"""

__author__ = """Joaquim Ramos de Carvalho"""
__email__ = 'joaquim@uc.pt'
__version__ = '0.3.9'
